import express from "express";

/**
 * Custom router class
 *
 * @export
 * @class Router
 * @abstract
 */
export class Router {
  /**
   * @type {express.Router}
   */
  #router;
  /**
   * @type {string}
   */
  #path;
  /**
   * Creates an instance of Router.
   * @constructor
   * @param {string} path
   */
  constructor(path) {
    this.#router = express.Router();
    this.#path = path;
  }

  /**
   * @public
   * @readonly
   * @type {express.Router}
   */
  get router() {
    return this.#router;
  }

  /**
   * @public
   * @readonly
   * @type {string}
   */
  get path() {
    return this.#path;
  }

  /**
   * @public
   * @param {Router} router
   * @returns {this}
   */
  subRoute(router) {
    this.#router.use(router.path, router.router);
    return this;
  }

  /**
   * @public
   * @param {string} path
   * @param {RequestHandler} handler
   * @param {...RequestHandler} middlewares
   * @returns {this}
   */
  get(path, handler, ...middlewares) {
    this.#router.get(path, ...middlewares, handler);
    return this;
  }
  /**
   * @public
   * @param {string} path
   * @param {RequestHandler} handler
   * @param {...RequestHandler} middlewares
   * @returns {this}
   */
  post(path, handler, ...middlewares) {
    this.#router.post(path, ...middlewares, handler);
    return this;
  }
  /**
   * @public
   * @param {string} path
   * @param {RequestHandler} handler
   * @param {...RequestHandler} middlewares
   * @returns {this}
   */
  put(path, handler, ...middlewares) {
    this.#router.put(path, ...middlewares, handler);
    return this;
  }
  /**
   * @public
   * @param {string} path
   * @param {RequestHandler} handler
   * @param {...RequestHandler} middlewares
   * @returns {this}
   */
  delete(path, handler, ...middlewares) {
    this.#router.delete(path, ...middlewares, handler);
    return this;
  }
  /**
   * @public
   * @param {string} path
   * @param {RequestHandler} handler
   * @param {...RequestHandler} middlewares
   * @returns {this}
   */
  patch(path, handler, ...middlewares) {
    this.#router.patch(path, ...middlewares, handler);
    return this;
  }
  /**
   * @public
   * @param {string} path
   * @param {RequestHandler} handler
   * @param {...RequestHandler} middlewares
   * @returns {this}
   */
  options(path, handler, ...middlewares) {
    this.#router.options(path, ...middlewares, handler);
    return this;
  }
  /**
   * @public
   * @param {string} path
   * @param {RequestHandler} handler
   * @param {...RequestHandler} middlewares
   * @returns {this}
   */
  head(path, handler, ...middlewares) {
    this.#router.head(path, ...middlewares, handler);
    return this;
  }
}

/**
 * @callback RequestHandler
 * @param {express.Request} req
 * @param {import('../middlewares/custom-response.middleware.mjs').Response} res
 * @param {express.NextFunction} [next]
 * @returns {void}
 */
